@@ -312,8 +312,6 @@ static void asyncio_loop() {
 	uwsgi.wait_write_hook = uwsgi_asyncio_wait_write_hook;
 	uwsgi.wait_read_hook = uwsgi_asyncio_wait_read_hook;
 
-	uwsgi.schedule_fix = uwsgi_asyncio_schedule_fix;
-
 	if (uwsgi.async < 2) {
 		uwsgi_log("the asyncio loop engine requires async mode (--async <n>)\n");
 		exit(1);
@@ -323,9 +321,20 @@ static void asyncio_loop() {
                 uwsgi_log("*** DANGER *** asyncio mode without coroutine/greenthread engine loaded !!!\n");
         }
 
+	if (!uwsgi.schedule_to_req) {
+		uwsgi.schedule_to_req = async_schedule_to_req_green;
+	}
+	else {
+		uwsgi.schedule_fix = uwsgi_asyncio_schedule_fix;
+	}
+
+#ifndef PYTHREE
+	PyObject *asyncio = PyImport_ImportModule("trollius");
+#else
 	PyObject *asyncio = PyImport_ImportModule("asyncio");
+#endif
 	if (!asyncio) uwsgi_pyexit;
-	
+
 	uasyncio.mod = asyncio;
 
 	uasyncio.loop = PyObject_CallMethod(asyncio, "get_event_loop", NULL);
