@@ -66,32 +66,14 @@ SV * coroae_coro_new(CV *block) {
 }
 
 static int coroae_wait_milliseconds(int timeout) {
-        int ret = -1;
-        dSP;
-        ENTER;
-        SAVETMPS;
-        PUSHMARK(SP);
-        XPUSHs(newSVnv(((double)timeout)/1000.0));
-        PUTBACK;
-        call_pv("Coro::AnyEvent::sleep", G_SCALAR|G_EVAL);
-        SPAGAIN;
-        if(SvTRUE(ERRSV)) {
-                uwsgi_log("[uwsgi-perl error] %s", SvPV_nolen(ERRSV));
-        }
-        else {
-                SV *p_ret = POPs;
-                if (SvTRUE(p_ret)) {
-                        ret = 0;
-                }
-        }
-        PUTBACK;
-        FREETMPS;
-        LEAVE;
-
-        return ret;
+	char buf[256];
+	double d = ((double)timeout)/1000.0;
+	int ret = snprintf(buf, 256, "Coro::AnyEvent::sleep %f", d);
+	if (ret <= 0 || ret > 256) return -1;
+	perl_eval_pv(buf, 0);
+	return 0;
 }
 
-
 static int coroae_wait_fd_read(int fd, int timeout) {
 	int ret = 0;
 	dSP;
