@@ -46,6 +46,8 @@ struct uwsgi_http {
 	int connect_timeout;
 	int manage_source;
 	int enable_proxy_protocol;
+	int chunked_input;
+	int manage_rtsp;
 }; 
 
 struct http_session {
@@ -134,6 +136,8 @@ struct http_session {
 
 	ssize_t (*func_write)(struct corerouter_peer *);
 
+	int is_rtsp;
+
 };
 
 
