@@ -65,7 +65,11 @@ struct uwsgi_option uwsgi_php_options[] = {
 };
 
 
+#ifdef UWSGI_PHP7
+static size_t sapi_uwsgi_ub_write(const char *str, size_t str_length TSRMLS_DC)
+#else
 static int sapi_uwsgi_ub_write(const char *str, uint str_length TSRMLS_DC)
+#endif
 {
 	struct wsgi_request *wsgi_req = (struct wsgi_request *) SG(server_context);
 
@@ -111,7 +115,11 @@ static int sapi_uwsgi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 	return SAPI_HEADER_SENT_SUCCESSFULLY;
 }
 
+#ifdef UWSGI_PHP7
+static size_t sapi_uwsgi_read_post(char *buffer, size_t count_bytes TSRMLS_DC)
+#else
 static int sapi_uwsgi_read_post(char *buffer, uint count_bytes TSRMLS_DC)
+#endif
 {
 	uint read_bytes = 0;
 	
@@ -235,7 +243,11 @@ PHP_MINIT_FUNCTION(uwsgi_php_minit) {
 }
 
 PHP_FUNCTION(uwsgi_version) {
+#ifdef UWSGI_PHP7
+	RETURN_STRING(UWSGI_VERSION);
+#else
 	RETURN_STRING(UWSGI_VERSION, 1);
+#endif
 }
 
 PHP_FUNCTION(uwsgi_worker_id) {
@@ -321,7 +333,11 @@ PHP_FUNCTION(uwsgi_cache_get) {
 	if (value) {
 		char *ret = estrndup(value, valsize);
 		free(value);
+#ifdef UWSGI_PHP7
+		RETURN_STRING(ret);
+#else
 		RETURN_STRING(ret, 0);
+#endif
 	}
 	RETURN_NULL();
 }
@@ -425,7 +441,11 @@ PHP_FUNCTION(uwsgi_rpc) {
 		// here we do not free varargs for performance reasons
 		char *ret = estrndup(response, size);
 		free(response);
+#ifdef UWSGI_PHP7
+		RETURN_STRING(ret);
+#else
 		RETURN_STRING(ret, 0);
+#endif
         }
 
 clear:
