@@ -160,6 +160,8 @@ struct uwsgi_option uwsgi_python_options[] = {
 	{"wsgi-accept-buffer", no_argument, 0, "accept CPython buffer-compliant objects as WSGI response in addition to string/bytes", uwsgi_opt_true, &up.wsgi_accept_buffer, 0},
 	{"wsgi-accept-buffers", no_argument, 0, "accept CPython buffer-compliant objects as WSGI response in addition to string/bytes", uwsgi_opt_true, &up.wsgi_accept_buffer, 0},
 
+	{"wsgi-disable-file-wrapper", no_argument, 0, "disable wsgi.file_wrapper feature", uwsgi_opt_true, &up.wsgi_disable_file_wrapper, 0},
+
 	{"python-version", no_argument, 0, "report python version", uwsgi_opt_pyver, NULL, UWSGI_OPT_IMMEDIATE},
 
 	{"python-raw", required_argument, 0, "load a python file for managing raw requests", uwsgi_opt_set_str, &up.raw, 0},
@@ -170,6 +172,8 @@ struct uwsgi_option uwsgi_python_options[] = {
 
 	{"py-call-osafterfork", no_argument, 0, "enable child processes running cpython to trap OS signals", uwsgi_opt_true, &up.call_osafterfork, 0},
 
+	{"python-worker-override", required_argument, 0, "override worker with the specified python script", uwsgi_opt_set_str, &up.worker_override, 0},
+
 	{0, 0, 0, 0, 0, 0, 0},
 };
 
@@ -792,6 +796,19 @@ void init_uwsgi_embedded_module() {
 		exit(1);
 	}
 
+	PyObject *py_sockets_list = PyList_New(0);
+	struct uwsgi_socket *uwsgi_sock = uwsgi.sockets;
+	while(uwsgi_sock) {
+		if (uwsgi_sock->bound) {
+			PyList_Append(py_sockets_list, PyInt_FromLong(uwsgi_sock->fd));
+		}
+		uwsgi_sock = uwsgi_sock->next;
+	}
+	if (PyDict_SetItemString(up.embedded_dict, "sockets", py_sockets_list)) {
+                PyErr_Print();
+                exit(1);
+        }
+
 	PyObject *py_magic_table = PyDict_New();
 	uint8_t mtk;
 	for (i = 0; i <= 0xff; i++) {
@@ -1626,6 +1643,26 @@ void uwsgi_python_add_item(char *key, uint16_t keylen, char *val, uint16_t valle
 	Py_DECREF(zero);
 }
 
+PyObject *uwsgi_python_dict_from_spooler_content(char *filename, char *buf, uint16_t len, char *body, size_t body_len) {
+
+	PyObject *spool_dict = PyDict_New();
+
+	PyObject *value = PyString_FromString(filename);
+	PyDict_SetItemString(spool_dict, "spooler_task_name", value);
+	Py_DECREF(value);
+
+	if (uwsgi_hooked_parse(buf, len, uwsgi_python_add_item, spool_dict))
+		return NULL;
+
+	if (body && body_len > 0) {
+		PyObject *value = PyString_FromStringAndSize(body, body_len);
+		PyDict_SetItemString(spool_dict, "body", value);
+		Py_DECREF(value);
+	}
+
+	return spool_dict;
+}
+
 int uwsgi_python_spooler(char *filename, char *buf, uint16_t len, char *body, size_t body_len) {
 
 	static int random_seed_reset = 0;
@@ -1651,25 +1688,15 @@ int uwsgi_python_spooler(char *filename, char *buf, uint16_t len, char *body, si
 	}
 
 	int retval = -1;
-	PyObject *spool_dict = PyDict_New();
 	PyObject *pyargs = PyTuple_New(1);
 	PyObject *ret = NULL;
 
-	PyObject *value = PyString_FromString(filename);
-	PyDict_SetItemString(spool_dict, "spooler_task_name", value);
-	Py_DECREF(value);
-
-	if (uwsgi_hooked_parse(buf, len, uwsgi_python_add_item, spool_dict)) {
-		// malformed packet, destroy it
+	PyObject *spool_dict = uwsgi_python_dict_from_spooler_content(filename, buf, len, body, body_len);
+	if (!spool_dict) {
 		retval = -2;
 		goto clear;
 	}
 
-	if (body && body_len > 0) {
-		PyObject *value = PyString_FromStringAndSize(body, body_len);
-		PyDict_SetItemString(spool_dict, "body", value);
-		Py_DECREF(value);
-	}
 	// PyTuple_SetItem steals a reference !!!
 	Py_INCREF(spool_dict);
 	PyTuple_SetItem(pyargs, 0, spool_dict);
@@ -1895,6 +1922,22 @@ static void uwsgi_python_on_load() {
 	uwsgi_register_logger("python", uwsgi_python_logger);
 }
 
+static int uwsgi_python_worker() {
+	if (!up.worker_override)
+		return 0;
+	UWSGI_GET_GIL;
+	// ensure signals can be used again from python
+	if (!up.call_osafterfork)
+		PyOS_AfterFork();
+	FILE *pyfile = fopen(up.worker_override, "r");
+	if (!pyfile) {
+		uwsgi_error_open(up.worker_override);
+		exit(1);
+	}
+	PyRun_SimpleFile(pyfile, up.worker_override);
+	return 1;
+}
+
 struct uwsgi_plugin python_plugin = {
 	.name = "python",
 	.alias = "python",
@@ -1946,5 +1989,6 @@ struct uwsgi_plugin python_plugin = {
 	.exception_log = uwsgi_python_exception_log,
 	.backtrace = uwsgi_python_backtrace,
 
+	.worker = uwsgi_python_worker,
 
 };
