@@ -32,9 +32,17 @@ if not 'UWSGI_PYTHON_NOLIB' in os.environ:
         # try 3.x style config dir
         if not os.path.exists(libdir):
             libdir = '%s/lib/python%s/config-%s' % (sys.prefix, version, get_python_version())
-        libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LIBRARY'))
-        if not os.path.exists(libpath): 
+
+        # get cpu type
+        uname = os.uname()
+        if uname[4].startswith('arm'):
+            libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LIBRARY'))
+            if not os.path.exists(libpath): 
+                libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LDLIBRARY'))
+        else:
             libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LDLIBRARY'))
+            if not os.path.exists(libpath): 
+                libpath = '%s/%s' % (libdir, sysconfig.get_config_var('LIBRARY'))
         if not os.path.exists(libpath): 
             libpath = '%s/libpython%s.a' % (libdir, version)
         LIBS.append(libpath)
@@ -43,11 +51,14 @@ if not 'UWSGI_PYTHON_NOLIB' in os.environ:
             LIBS.append('-lutil')
     else:
         try:
-            LDFLAGS.append("-L%s" % sysconfig.get_config_var('LIBDIR'))
-            os.environ['LD_RUN_PATH'] = "%s" % (sysconfig.get_config_var('LIBDIR'))
+            libdir = sysconfig.get_config_var('LIBDIR')
         except:
-            LDFLAGS.append("-L%s/lib" % sysconfig.PREFIX)
-            os.environ['LD_RUN_PATH'] = "%s/lib" % sysconfig.PREFIX
+            libdir = "%s/lib" % sysconfig.PREFIX
+
+        LDFLAGS.append("-L%s" % libdir)
+        LDFLAGS.append("-Wl,-rpath=%s" % libdir)
+
+        os.environ['LD_RUN_PATH'] = "%s" % libdir
 
         LIBS.append('-lpython%s' % get_python_version())
 else:
